import 'dart:html';
import 'dart:typed_data';
import 'package:rad_textures/rad_textures.dart';
import 'package:vector_math/vector_math.dart';

class Texture2D {
  int _width;
  Float32List _buffer;
  Texture2D(int width) {
    this._width = width;
    _buffer = new Float32List(width * width);
  }
  int offset(int x, int y) {
    return (y * _width) + x;
  }
  void set(int x, int y, double val) {
    assert(x < _width);
    assert(y < _width);
    _buffer[offset(x, y)] = val;
  }
  double get(int x, int y) {
    assert(x < _width);
    assert(y < _width);
    return _buffer[offset(x, y)];
  }
  Iterable<int> offsets() sync* {
    for(int y = 0; y < _width; y++)
      for(int x = 0; x < _width; x++)
        yield (y * _width) + x;
  }
  void normalize() {
    double min = double.MAX_FINITE;
    double max = -double.MAX_FINITE;
    for(int offset in offsets()) {
      double val = _buffer[offset];
      if(val < min) min = val;
      if(val > max) max = val;
    }
    double height = max - min;
    for(int offset in offsets()) {
      double sample = _buffer[offset];
      sample -= min;
      sample /= height;
      _buffer[offset] = sample;
    }
  }
}

void main() {
  window.onLoad.listen((e) {
    _plot_noise("canvas");
  });
}

int width = 512;

CanvasElement _append_canvas(String label) {
  DivElement div = document.getElementById("noises_pane");
  CanvasElement canvas = document.createElement("canvas");
  canvas.width = width;
  canvas.height = width;
  div.append(canvas);
  div.append(document.createElement("br"));
  SpanElement span = document.createElement("span");
  span.innerHtml = label;
  div.append(span);
  div.append(document.createElement("br"));
  return canvas;
}

double scale = 0.03;

void _plot_noise_2(String canvas_id) {
  CanvasElement canvas = _append_canvas(canvas_id);
  CanvasRenderingContext2D ctx = canvas.getContext('2d');
  ImageData imageData = ctx.getImageData(0, 0, canvas.width, canvas.height);
  Texture2D texture = new Texture2D(width);
  for (int x = 0; x < width; x++) {
    for (int y = 0; y < width; y++) {
      double sample = worley1(x * scale, y * scale, 0.0, 8.0)[0].distance;
      texture.set(x, y, sample);
    }
  }
  texture.normalize();
  for (int x = 0; x < width; x++) {
    for (int y = 0; y < width; y++) {
      double sample = texture.get(x, y);
      imageData.data[(y * width + x) * 4 + 0] = 0;
      //if(sample < 0.6) sample = 0.0;
      imageData.data[(y * width + x) * 4 + 1] = FASTFLOOR((sample * 0.5 + 0.5) * 255);
      imageData.data[(y * width + x) * 4 + 2] = 0;
      imageData.data[(y * width + x) * 4 + 3] = 255;
    }
  }
  ctx.putImageData(imageData, 0, 0, 0, 0, canvas.width, canvas.height);
}

int frac_int(double val, int scale) {
  double base = val.floor().toDouble();
  double frac = val - base;
  frac *= scale;
  return frac.toInt();
}

void _plot_noise(String canvas_id) {
  CanvasElement canvas = _append_canvas(canvas_id);
  CanvasRenderingContext2D ctx = canvas.getContext('2d');
  ImageData imageData = ctx.getImageData(0, 0, canvas.width, canvas.height);
  for (int x = 0; x < width; x++) {
    for (int y = 0; y < width; y++) {
      var center = worley1(x * scale, y * scale, 0.0, 8.0)[0];
      imageData.data[(y * width + x) * 4 + 0] = frac_int(center.v_center.x, 255);
      imageData.data[(y * width + x) * 4 + 1] = frac_int(center.v_center.y, 255);
      imageData.data[(y * width + x) * 4 + 2] = frac_int(center.v_center.z, 255);
      imageData.data[(y * width + x) * 4 + 3] = 255;
    }
  }
  ctx.putImageData(imageData, 0, 0, 0, 0, canvas.width, canvas.height);
}
